@@ -1,11 +1,11 @@
 from abc import ABC, abstractmethod
 from .agent import AgentChatContext
 from jsonschema import validate
-from meshagent.tools.toolkit import Response, Toolkit
-from meshagent.api import RoomClient
+from meshagent.tools import Response, Toolkit, ToolkitBuilder, ToolkitConfig
+from meshagent.api import RoomClient, RoomException, RemoteParticipant
 from typing import Any, Optional, Callable, TypeVar, Generic
 
-T = TypeVar("T")
+TEvent = TypeVar("T")
 
 
 class ToolResponseAdapter(ABC):
@@ -28,7 +28,10 @@ class ToolResponseAdapter(ABC):
         pass
 
 
-class LLMAdapter(Generic[T]):
+class LLMAdapter(Generic[TEvent]):
+    @abstractmethod
+    def default_model(self) -> str: ...
+
     def create_chat_context(self) -> AgentChatContext:
         return AgentChatContext()
 
@@ -38,6 +41,18 @@ class LLMAdapter(Generic[T]):
     ):
         return True
 
+    def tool_providers(self, *, model: str) -> list[ToolkitBuilder]:
+        return []
+
+    def make_toolkit(self, *, model: str, config: ToolkitConfig) -> Toolkit:
+        for tool in self.tool_providers(model=model):
+            if tool.name == config.name:
+                return Toolkit(
+                    name=config.name, tools=[tool.make(model=model, config=config)]
+                )
+
+        raise RoomException(f"Unexpected tool: {config.name} for model {model}")
+
     @abstractmethod
     async def next(
         self,
@@ -47,7 +62,9 @@ class LLMAdapter(Generic[T]):
         toolkits: list[Toolkit],
         tool_adapter: Optional[ToolResponseAdapter] = None,
         output_schema: Optional[dict] = None,
-        event_handler: Optional[Callable[[T], None]] = None,
+        event_handler: Optional[Callable[[TEvent], None]] = None,
+        model: Optional[str] = None,
+        on_behalf_of: Optional[RemoteParticipant] = None,
     ) -> Any:
         pass
 
