@@ -12,16 +12,13 @@ from meshagent.api import (
     ToolkitDescription,
     Participant,
     RemoteParticipant,
-    meshagent_base_url,
     StorageEntry,
 )
 from meshagent.api.protocol import Protocol
-from meshagent.tools.toolkit import (
+from meshagent.tools import (
     Toolkit,
     Tool,
     ToolContext,
-    toolkit_factory,
-    register_toolkit_factory,
 )
 from meshagent.api.room_server_client import RoomClient
 from jsonschema import validate
@@ -194,52 +191,59 @@ class SingleRoomAgent(Agent):
 
         installed = False
 
+        builtin_agents_url = "http://localhost:8080"
+
         for requirement in self.requires:
-            if isinstance(requirement, RequiredToolkit):
-                if toolkit_factory(requirement.name) is not None:
-                    # no need to install something we can create from a factory
-                    continue
+            if requirement.callable:
+                if isinstance(requirement, RequiredToolkit):
+                    if requirement.name == "ui":
+                        # TODO: maybe requirements can be marked as non installable?
+                        continue
 
-                if requirement.name == "ui":
-                    # TODO: maybe requirements can be marked as non installable?
-                    continue
+                    if requirement.name not in toolkits_by_name:
+                        installed = True
 
-                if requirement.name not in toolkits_by_name:
-                    installed = True
+                        logger.info(
+                            f"calling required tool into room {requirement.name}"
+                        )
 
-                    logger.info(f"Installing required tool {requirement.name}")
+                        if requirement.name.startswith(
+                            "https://"
+                        ) or requirement.name.startswith("http://"):
+                            url = requirement.name
+                        else:
+                            url = f"{builtin_agents_url}/toolkits/{requirement.name}"
 
-                    if requirement.name.startswith("https://"):
-                        url = requirement.name
-                    else:
-                        url = f"{meshagent_base_url()}/toolkits/{requirement.name}"
+                        await self._room.agents.make_call(
+                            url=url, name=requirement.name, arguments={}
+                        )
 
-                    await self._room.agents.make_call(
-                        url=url, name=requirement.name, arguments={}
-                    )
+                elif isinstance(requirement, RequiredSchema):
+                    if requirement.name not in schemas_by_name:
+                        installed = True
 
-            elif isinstance(requirement, RequiredSchema):
-                if requirement.name not in schemas_by_name:
-                    installed = True
+                        logger.info(f"Installing required schema {requirement.name}")
 
-                    logger.info(f"Installing required schema {requirement.name}")
+                        if requirement.name.startswith(
+                            "https://"
+                        ) or requirement.name.startswith("http://"):
+                            url = requirement.name
+                        else:
+                            url = f"{builtin_agents_url}/schemas/{requirement.name}"
 
-                    if requirement.name.startswith("https://"):
-                        url = requirement.name
-                    else:
-                        url = f"{meshagent_base_url()}/schemas/{requirement.name}"
+                        await self._room.agents.make_call(
+                            url=url, name=requirement.name, arguments={}
+                        )
 
-                    await self._room.agents.make_call(
-                        url=url, name=requirement.name, arguments={}
-                    )
-
-            else:
-                raise RoomException("unsupported requirement")
+                else:
+                    raise RoomException("unsupported requirement")
 
         if installed:
             await asyncio.sleep(5)
 
-    async def get_required_toolkits(self, context: ToolContext) -> list[Toolkit]:
+    async def get_toolkits(
+        self, context: ToolContext, remote_toolkits: list[RequiredToolkit]
+    ):
         tool_target = context.caller
         if context.on_behalf_of is not None:
             tool_target = context.on_behalf_of
@@ -255,20 +259,18 @@ class SingleRoomAgent(Agent):
         for toolkit_description in visible_tools:
             toolkits_by_name[toolkit_description.name] = toolkit_description
 
-        for required_toolkit in self.requires:
+        for required_toolkit in remote_toolkits:
             if isinstance(required_toolkit, RequiredToolkit):
-                if toolkit_factory(required_toolkit.name) is not None:
-                    toolkit = await toolkit_factory(required_toolkit.name)(
-                        context, required_toolkit
-                    )
-                    toolkits.append(toolkit)
-                    continue
-
                 toolkit = toolkits_by_name.get(required_toolkit.name, None)
                 if toolkit is None:
-                    raise RoomException(
-                        f"unable to locate required toolkit {required_toolkit.name}"
-                    )
+                    if context.on_behalf_of is not None:
+                        raise RoomException(
+                            f"unable to get toolkit {required_toolkit.name} on behalf of {context.on_behalf_of}"
+                        )
+                    else:
+                        raise RoomException(
+                            f"unable to get toolkit {required_toolkit.name} for caller {context.caller.id}"
+                        )
 
                 room_tools = list[RoomTool]()
 
@@ -296,7 +298,7 @@ class SingleRoomAgent(Agent):
                         tool_description = tools_by_name.get(required_tool, None)
                         if tool_description is None:
                             raise RoomException(
-                                f"unable to locate required tool {required_tool}"
+                                f"unable to locate required tool {required_tool} in toolkit {required_toolkit.name}"
                             )
 
                         tool = RoomTool(
@@ -324,6 +326,9 @@ class SingleRoomAgent(Agent):
 
         return toolkits
 
+    async def get_required_toolkits(self, context: ToolContext) -> list[Toolkit]:
+        return await self.get_toolkits(context, self.requires)
+
 
 class TaskRunner(SingleRoomAgent):
     def __init__(
@@ -624,6 +629,3 @@ async def make_run_task_tool(context: ToolContext, toolkit: RequiredToolkit):
         )
 
     return Toolkit(name="agents", tools=tools)
-
-
-register_toolkit_factory("agents", make_run_task_tool)
