@@ -2,8 +2,10 @@ from .agent import SingleRoomAgent
 from meshagent.api import websocket_protocol, RoomClient
 import asyncio
 import signal
+from warnings import deprecated
 
 
+@deprecated("use ServiceHost and the cli to connect agents to the room")
 async def connect_development_agent(*, room_name: str, agent: SingleRoomAgent):
     async with RoomClient(
         protocol=websocket_protocol(
@@ -22,7 +24,7 @@ async def connect_development_agent(*, room_name: str, agent: SingleRoomAgent):
             signal.signal(signal.SIGABRT, clean_termination)
 
             await asyncio.wait(
-                [asyncio.ensure_future(room.protocol.wait_for_close()), term],
+                [asyncio.create_task(room.protocol.wait_for_close()), term],
                 return_when=asyncio.FIRST_COMPLETED,
             )
 
