@@ -1,7 +1,7 @@
 from meshagent.agents.agent import AgentCallContext, AgentException
 from meshagent.api import Requirement
 from meshagent.api.messaging import TextResponse
-from meshagent.tools.toolkit import Toolkit, Tool, ToolContext
+from meshagent.tools import Toolkit, Tool, ToolContext
 from meshagent.api.schema import MeshSchema
 from meshagent.agents.writer import Writer, WriterContext
 from meshagent.agents.adapter import LLMAdapter, ToolResponseAdapter
@@ -247,7 +247,7 @@ class PlanningWriter(Writer):
             i += 1
 
             try:
-                logger.info("COMPLETION STARTING: Step %s", i)
+                logger.info("Working on step %s", i)
 
                 base_args = arguments.copy()
                 base_args.pop("path")
@@ -598,7 +598,7 @@ class DynamicPlanningResponder(TaskRunner):
             i += 1
 
             try:
-                logger.info("COMPLETION STARTING: Step %s", i)
+                logger.info("Working on step %s", i)
 
                 toolkits = [*self.toolkits, *context.toolkits]
 
